import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Optional, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.event_trigger_event_type import EventTriggerEventType
from ..models.event_trigger_severity import EventTriggerSeverity
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.auto_resolve_event_trigger_condition import AutoResolveEventTriggerCondition
    from ..models.base_event_trigger_predicate import BaseEventTriggerPredicate
    from ..models.battery_event_trigger_condition import BatteryEventTriggerCondition
    from ..models.bitset_event_trigger_condition import BitsetEventTriggerCondition
    from ..models.event_trigger_command import EventTriggerCommand
    from ..models.event_trigger_sms_tags import EventTriggerSmsTags
    from ..models.event_trigger_tags import EventTriggerTags
    from ..models.forwarding_configuration import ForwardingConfiguration
    from ..models.json_event_trigger_condition import JsonEventTriggerCondition
    from ..models.numeric_set_event_trigger_condition import NumericSetEventTriggerCondition
    from ..models.presence_event_trigger_condition import PresenceEventTriggerCondition
    from ..models.regex_event_trigger_condition import RegexEventTriggerCondition
    from ..models.scope_filter import ScopeFilter
    from ..models.stateful_trigger_configuration import StatefulTriggerConfiguration
    from ..models.threshold_event_trigger_condition import ThresholdEventTriggerCondition
    from ..models.triggered_configuration import TriggeredConfiguration


T = TypeVar("T", bound="EventTrigger")


@attr.s(auto_attribs=True)
class EventTrigger:
    """
    Attributes:
        event_type (EventTriggerEventType):
        message (str):
        interval (int):
        severity (EventTriggerSeverity):
        commands (List['EventTriggerCommand']):
        notification_enabled (bool):
        organization_id (Union[Unset, str]):
        condition (Union['AutoResolveEventTriggerCondition', 'BatteryEventTriggerCondition',
            'BitsetEventTriggerCondition', 'JsonEventTriggerCondition', 'NumericSetEventTriggerCondition',
            'PresenceEventTriggerCondition', 'RegexEventTriggerCondition', 'ThresholdEventTriggerCondition', None]):
        exit_condition (Union['AutoResolveEventTriggerCondition', 'BatteryEventTriggerCondition',
            'BitsetEventTriggerCondition', 'JsonEventTriggerCondition', 'NumericSetEventTriggerCondition',
            'PresenceEventTriggerCondition', 'RegexEventTriggerCondition', 'ThresholdEventTriggerCondition', None]):
        predicate (Union['BaseEventTriggerPredicate', None]):
        enabled (Union[Unset, bool]):
        format_ (Union[Unset, str]):
        triggered_configuration (Optional[TriggeredConfiguration]):
        sms_tags (Union[Unset, EventTriggerSmsTags]):
        device_scope (Union[Unset, None, ScopeFilter]):
        last_triggered_time (Union[Unset, None, datetime.datetime]):
        event_trigger_group_id (Union[Unset, None, str]):
        stateful_trigger_configuration (Optional[StatefulTriggerConfiguration]):
        is_stateful (Union[Unset, bool]):
        forwarding_configuration (Optional[ForwardingConfiguration]):
        sets_device_color (Union[Unset, bool]):
        id (Union[Unset, str]):
        created_at (Union[Unset, datetime.datetime]):
        updated_at (Union[Unset, datetime.datetime]):
        tags (Union[Unset, EventTriggerTags]):
    """

    event_type: EventTriggerEventType
    message: str
    interval: int
    severity: EventTriggerSeverity
    commands: List["EventTriggerCommand"]
    notification_enabled: bool
    condition: Union[
        "AutoResolveEventTriggerCondition",
        "BatteryEventTriggerCondition",
        "BitsetEventTriggerCondition",
        "JsonEventTriggerCondition",
        "NumericSetEventTriggerCondition",
        "PresenceEventTriggerCondition",
        "RegexEventTriggerCondition",
        "ThresholdEventTriggerCondition",
        None,
    ]
    exit_condition: Union[
        "AutoResolveEventTriggerCondition",
        "BatteryEventTriggerCondition",
        "BitsetEventTriggerCondition",
        "JsonEventTriggerCondition",
        "NumericSetEventTriggerCondition",
        "PresenceEventTriggerCondition",
        "RegexEventTriggerCondition",
        "ThresholdEventTriggerCondition",
        None,
    ]
    predicate: Union["BaseEventTriggerPredicate", None]
    triggered_configuration: Optional["TriggeredConfiguration"]
    stateful_trigger_configuration: Optional["StatefulTriggerConfiguration"]
    forwarding_configuration: Optional["ForwardingConfiguration"]
    organization_id: Union[Unset, str] = UNSET
    enabled: Union[Unset, bool] = UNSET
    format_: Union[Unset, str] = UNSET
    sms_tags: Union[Unset, "EventTriggerSmsTags"] = UNSET
    device_scope: Union[Unset, None, "ScopeFilter"] = UNSET
    last_triggered_time: Union[Unset, None, datetime.datetime] = UNSET
    event_trigger_group_id: Union[Unset, None, str] = UNSET
    is_stateful: Union[Unset, bool] = UNSET
    sets_device_color: Union[Unset, bool] = UNSET
    id: Union[Unset, str] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    updated_at: Union[Unset, datetime.datetime] = UNSET
    tags: Union[Unset, "EventTriggerTags"] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from ..models.base_event_trigger_predicate import BaseEventTriggerPredicate
        from ..models.battery_event_trigger_condition import BatteryEventTriggerCondition
        from ..models.bitset_event_trigger_condition import BitsetEventTriggerCondition
        from ..models.json_event_trigger_condition import JsonEventTriggerCondition
        from ..models.numeric_set_event_trigger_condition import NumericSetEventTriggerCondition
        from ..models.presence_event_trigger_condition import PresenceEventTriggerCondition
        from ..models.regex_event_trigger_condition import RegexEventTriggerCondition
        from ..models.threshold_event_trigger_condition import ThresholdEventTriggerCondition

        event_type = self.event_type.value

        message = self.message
        interval = self.interval
        severity = self.severity.value

        commands = []
        for commands_item_data in self.commands:
            commands_item = commands_item_data.to_dict()

            commands.append(commands_item)

        notification_enabled = self.notification_enabled
        organization_id = self.organization_id
        condition: Union[Dict[str, Any], None]
        if self.condition is None:
            condition = None

        elif isinstance(self.condition, PresenceEventTriggerCondition):
            condition = self.condition.to_dict()

        elif isinstance(self.condition, ThresholdEventTriggerCondition):
            condition = self.condition.to_dict()

        elif isinstance(self.condition, RegexEventTriggerCondition):
            condition = self.condition.to_dict()

        elif isinstance(self.condition, BitsetEventTriggerCondition):
            condition = self.condition.to_dict()

        elif isinstance(self.condition, BatteryEventTriggerCondition):
            condition = self.condition.to_dict()

        elif isinstance(self.condition, NumericSetEventTriggerCondition):
            condition = self.condition.to_dict()

        elif isinstance(self.condition, JsonEventTriggerCondition):
            condition = self.condition.to_dict()

        else:
            condition = self.condition.to_dict()

        exit_condition: Union[Dict[str, Any], None]
        if self.exit_condition is None:
            exit_condition = None

        elif isinstance(self.exit_condition, PresenceEventTriggerCondition):
            exit_condition = self.exit_condition.to_dict()

        elif isinstance(self.exit_condition, ThresholdEventTriggerCondition):
            exit_condition = self.exit_condition.to_dict()

        elif isinstance(self.exit_condition, RegexEventTriggerCondition):
            exit_condition = self.exit_condition.to_dict()

        elif isinstance(self.exit_condition, BitsetEventTriggerCondition):
            exit_condition = self.exit_condition.to_dict()

        elif isinstance(self.exit_condition, BatteryEventTriggerCondition):
            exit_condition = self.exit_condition.to_dict()

        elif isinstance(self.exit_condition, NumericSetEventTriggerCondition):
            exit_condition = self.exit_condition.to_dict()

        elif isinstance(self.exit_condition, JsonEventTriggerCondition):
            exit_condition = self.exit_condition.to_dict()

        else:
            exit_condition = self.exit_condition.to_dict()

        predicate: Union[Dict[str, Any], None]
        if self.predicate is None:
            predicate = None

        elif isinstance(self.predicate, BaseEventTriggerPredicate):
            predicate = self.predicate.to_dict()

        elif isinstance(self.predicate, BaseEventTriggerPredicate):
            predicate = self.predicate.to_dict()

        elif isinstance(self.predicate, BaseEventTriggerPredicate):
            predicate = self.predicate.to_dict()

        elif isinstance(self.predicate, BaseEventTriggerPredicate):
            predicate = self.predicate.to_dict()

        elif isinstance(self.predicate, BaseEventTriggerPredicate):
            predicate = self.predicate.to_dict()

        elif isinstance(self.predicate, BaseEventTriggerPredicate):
            predicate = self.predicate.to_dict()

        elif isinstance(self.predicate, BaseEventTriggerPredicate):
            predicate = self.predicate.to_dict()

        elif isinstance(self.predicate, BaseEventTriggerPredicate):
            predicate = self.predicate.to_dict()

        elif isinstance(self.predicate, BaseEventTriggerPredicate):
            predicate = self.predicate.to_dict()

        elif isinstance(self.predicate, BaseEventTriggerPredicate):
            predicate = self.predicate.to_dict()

        elif isinstance(self.predicate, BaseEventTriggerPredicate):
            predicate = self.predicate.to_dict()

        elif isinstance(self.predicate, BaseEventTriggerPredicate):
            predicate = self.predicate.to_dict()

        elif isinstance(self.predicate, BaseEventTriggerPredicate):
            predicate = self.predicate.to_dict()

        elif isinstance(self.predicate, BaseEventTriggerPredicate):
            predicate = self.predicate.to_dict()

        elif isinstance(self.predicate, BaseEventTriggerPredicate):
            predicate = self.predicate.to_dict()

        elif isinstance(self.predicate, BaseEventTriggerPredicate):
            predicate = self.predicate.to_dict()

        else:
            predicate = self.predicate.to_dict()

        enabled = self.enabled
        format_ = self.format_
        triggered_configuration = self.triggered_configuration.to_dict() if self.triggered_configuration else None

        sms_tags: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.sms_tags, Unset):
            sms_tags = self.sms_tags.to_dict()

        device_scope: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self.device_scope, Unset):
            device_scope = self.device_scope.to_dict() if self.device_scope else None

        last_triggered_time: Union[Unset, None, str] = UNSET
        if not isinstance(self.last_triggered_time, Unset):
            last_triggered_time = self.last_triggered_time.isoformat() if self.last_triggered_time else None

        event_trigger_group_id = self.event_trigger_group_id
        stateful_trigger_configuration = (
            self.stateful_trigger_configuration.to_dict() if self.stateful_trigger_configuration else None
        )

        is_stateful = self.is_stateful
        forwarding_configuration = self.forwarding_configuration.to_dict() if self.forwarding_configuration else None

        sets_device_color = self.sets_device_color
        id = self.id
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        updated_at: Union[Unset, str] = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()

        tags: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = self.tags.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "eventType": event_type,
                "message": message,
                "interval": interval,
                "severity": severity,
                "commands": commands,
                "notificationEnabled": notification_enabled,
                "condition": condition,
                "exitCondition": exit_condition,
                "predicate": predicate,
                "triggeredConfiguration": triggered_configuration,
                "statefulTriggerConfiguration": stateful_trigger_configuration,
                "forwardingConfiguration": forwarding_configuration,
            }
        )
        if organization_id is not UNSET:
            field_dict["organizationId"] = organization_id
        if enabled is not UNSET:
            field_dict["enabled"] = enabled
        if format_ is not UNSET:
            field_dict["format"] = format_
        if sms_tags is not UNSET:
            field_dict["smsTags"] = sms_tags
        if device_scope is not UNSET:
            field_dict["deviceScope"] = device_scope
        if last_triggered_time is not UNSET:
            field_dict["lastTriggeredTime"] = last_triggered_time
        if event_trigger_group_id is not UNSET:
            field_dict["eventTriggerGroupId"] = event_trigger_group_id
        if is_stateful is not UNSET:
            field_dict["isStateful"] = is_stateful
        if sets_device_color is not UNSET:
            field_dict["setsDeviceColor"] = sets_device_color
        if id is not UNSET:
            field_dict["id"] = id
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if updated_at is not UNSET:
            field_dict["updatedAt"] = updated_at
        if tags is not UNSET:
            field_dict["tags"] = tags

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.auto_resolve_event_trigger_condition import AutoResolveEventTriggerCondition
        from ..models.base_event_trigger_predicate import BaseEventTriggerPredicate
        from ..models.battery_event_trigger_condition import BatteryEventTriggerCondition
        from ..models.bitset_event_trigger_condition import BitsetEventTriggerCondition
        from ..models.event_trigger_command import EventTriggerCommand
        from ..models.event_trigger_sms_tags import EventTriggerSmsTags
        from ..models.event_trigger_tags import EventTriggerTags
        from ..models.forwarding_configuration import ForwardingConfiguration
        from ..models.json_event_trigger_condition import JsonEventTriggerCondition
        from ..models.numeric_set_event_trigger_condition import NumericSetEventTriggerCondition
        from ..models.presence_event_trigger_condition import PresenceEventTriggerCondition
        from ..models.regex_event_trigger_condition import RegexEventTriggerCondition
        from ..models.scope_filter import ScopeFilter
        from ..models.stateful_trigger_configuration import StatefulTriggerConfiguration
        from ..models.threshold_event_trigger_condition import ThresholdEventTriggerCondition
        from ..models.triggered_configuration import TriggeredConfiguration

        d = src_dict.copy()
        event_type = EventTriggerEventType(d.pop("eventType"))

        message = d.pop("message")

        interval = d.pop("interval")

        severity = EventTriggerSeverity(d.pop("severity"))

        commands = []
        _commands = d.pop("commands")
        for commands_item_data in _commands:
            commands_item = EventTriggerCommand.from_dict(commands_item_data)

            commands.append(commands_item)

        notification_enabled = d.pop("notificationEnabled")

        organization_id = d.pop("organizationId", UNSET)

        def _parse_condition(
            data: object,
        ) -> Union[
            "AutoResolveEventTriggerCondition",
            "BatteryEventTriggerCondition",
            "BitsetEventTriggerCondition",
            "JsonEventTriggerCondition",
            "NumericSetEventTriggerCondition",
            "PresenceEventTriggerCondition",
            "RegexEventTriggerCondition",
            "ThresholdEventTriggerCondition",
            None,
        ]:
            if data is None:
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                condition_type_0 = PresenceEventTriggerCondition.from_dict(data)

                return condition_type_0
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                condition_type_1 = ThresholdEventTriggerCondition.from_dict(data)

                return condition_type_1
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                condition_type_2 = RegexEventTriggerCondition.from_dict(data)

                return condition_type_2
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                condition_type_3 = BitsetEventTriggerCondition.from_dict(data)

                return condition_type_3
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                condition_type_4 = BatteryEventTriggerCondition.from_dict(data)

                return condition_type_4
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                condition_type_5 = NumericSetEventTriggerCondition.from_dict(data)

                return condition_type_5
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                condition_type_6 = JsonEventTriggerCondition.from_dict(data)

                return condition_type_6
            except:  # noqa: E722
                pass
            if not isinstance(data, dict):
                raise TypeError()
            condition_type_7 = AutoResolveEventTriggerCondition.from_dict(data)

            return condition_type_7

        condition = _parse_condition(d.pop("condition"))

        def _parse_exit_condition(
            data: object,
        ) -> Union[
            "AutoResolveEventTriggerCondition",
            "BatteryEventTriggerCondition",
            "BitsetEventTriggerCondition",
            "JsonEventTriggerCondition",
            "NumericSetEventTriggerCondition",
            "PresenceEventTriggerCondition",
            "RegexEventTriggerCondition",
            "ThresholdEventTriggerCondition",
            None,
        ]:
            if data is None:
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                exit_condition_type_0 = PresenceEventTriggerCondition.from_dict(data)

                return exit_condition_type_0
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                exit_condition_type_1 = ThresholdEventTriggerCondition.from_dict(data)

                return exit_condition_type_1
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                exit_condition_type_2 = RegexEventTriggerCondition.from_dict(data)

                return exit_condition_type_2
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                exit_condition_type_3 = BitsetEventTriggerCondition.from_dict(data)

                return exit_condition_type_3
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                exit_condition_type_4 = BatteryEventTriggerCondition.from_dict(data)

                return exit_condition_type_4
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                exit_condition_type_5 = NumericSetEventTriggerCondition.from_dict(data)

                return exit_condition_type_5
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                exit_condition_type_6 = JsonEventTriggerCondition.from_dict(data)

                return exit_condition_type_6
            except:  # noqa: E722
                pass
            if not isinstance(data, dict):
                raise TypeError()
            exit_condition_type_7 = AutoResolveEventTriggerCondition.from_dict(data)

            return exit_condition_type_7

        exit_condition = _parse_exit_condition(d.pop("exitCondition"))

        def _parse_predicate(data: object) -> Union["BaseEventTriggerPredicate", None]:
            if data is None:
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                predicate_type_0 = BaseEventTriggerPredicate.from_dict(data)

                return predicate_type_0
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                predicate_type_1 = BaseEventTriggerPredicate.from_dict(data)

                return predicate_type_1
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                predicate_type_2 = BaseEventTriggerPredicate.from_dict(data)

                return predicate_type_2
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                predicate_type_3 = BaseEventTriggerPredicate.from_dict(data)

                return predicate_type_3
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                predicate_type_4 = BaseEventTriggerPredicate.from_dict(data)

                return predicate_type_4
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                predicate_type_5 = BaseEventTriggerPredicate.from_dict(data)

                return predicate_type_5
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                predicate_type_6 = BaseEventTriggerPredicate.from_dict(data)

                return predicate_type_6
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                predicate_type_7 = BaseEventTriggerPredicate.from_dict(data)

                return predicate_type_7
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                predicate_type_8 = BaseEventTriggerPredicate.from_dict(data)

                return predicate_type_8
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                predicate_type_9 = BaseEventTriggerPredicate.from_dict(data)

                return predicate_type_9
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                predicate_type_10 = BaseEventTriggerPredicate.from_dict(data)

                return predicate_type_10
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                predicate_type_11 = BaseEventTriggerPredicate.from_dict(data)

                return predicate_type_11
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                predicate_type_12 = BaseEventTriggerPredicate.from_dict(data)

                return predicate_type_12
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                predicate_type_13 = BaseEventTriggerPredicate.from_dict(data)

                return predicate_type_13
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                predicate_type_14 = BaseEventTriggerPredicate.from_dict(data)

                return predicate_type_14
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                predicate_type_15 = BaseEventTriggerPredicate.from_dict(data)

                return predicate_type_15
            except:  # noqa: E722
                pass
            if not isinstance(data, dict):
                raise TypeError()
            predicate_type_16 = BaseEventTriggerPredicate.from_dict(data)

            return predicate_type_16

        predicate = _parse_predicate(d.pop("predicate"))

        enabled = d.pop("enabled", UNSET)

        format_ = d.pop("format", UNSET)

        _triggered_configuration = d.pop("triggeredConfiguration")
        triggered_configuration: Optional[TriggeredConfiguration]
        if _triggered_configuration is None:
            triggered_configuration = None
        else:
            triggered_configuration = TriggeredConfiguration.from_dict(_triggered_configuration)

        _sms_tags = d.pop("smsTags", UNSET)
        sms_tags: Union[Unset, EventTriggerSmsTags]
        if isinstance(_sms_tags, Unset):
            sms_tags = UNSET
        else:
            sms_tags = EventTriggerSmsTags.from_dict(_sms_tags)

        _device_scope = d.pop("deviceScope", UNSET)
        device_scope: Union[Unset, None, ScopeFilter]
        if _device_scope is None:
            device_scope = None
        elif isinstance(_device_scope, Unset):
            device_scope = UNSET
        else:
            device_scope = ScopeFilter.from_dict(_device_scope)

        _last_triggered_time = d.pop("lastTriggeredTime", UNSET)
        last_triggered_time: Union[Unset, None, datetime.datetime]
        if _last_triggered_time is None:
            last_triggered_time = None
        elif isinstance(_last_triggered_time, Unset):
            last_triggered_time = UNSET
        else:
            last_triggered_time = isoparse(_last_triggered_time)

        event_trigger_group_id = d.pop("eventTriggerGroupId", UNSET)

        _stateful_trigger_configuration = d.pop("statefulTriggerConfiguration")
        stateful_trigger_configuration: Optional[StatefulTriggerConfiguration]
        if _stateful_trigger_configuration is None:
            stateful_trigger_configuration = None
        else:
            stateful_trigger_configuration = StatefulTriggerConfiguration.from_dict(_stateful_trigger_configuration)

        is_stateful = d.pop("isStateful", UNSET)

        _forwarding_configuration = d.pop("forwardingConfiguration")
        forwarding_configuration: Optional[ForwardingConfiguration]
        if _forwarding_configuration is None:
            forwarding_configuration = None
        else:
            forwarding_configuration = ForwardingConfiguration.from_dict(_forwarding_configuration)

        sets_device_color = d.pop("setsDeviceColor", UNSET)

        id = d.pop("id", UNSET)

        _created_at = d.pop("createdAt", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        _updated_at = d.pop("updatedAt", UNSET)
        updated_at: Union[Unset, datetime.datetime]
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        _tags = d.pop("tags", UNSET)
        tags: Union[Unset, EventTriggerTags]
        if isinstance(_tags, Unset):
            tags = UNSET
        else:
            tags = EventTriggerTags.from_dict(_tags)

        event_trigger = cls(
            event_type=event_type,
            message=message,
            interval=interval,
            severity=severity,
            commands=commands,
            notification_enabled=notification_enabled,
            organization_id=organization_id,
            condition=condition,
            exit_condition=exit_condition,
            predicate=predicate,
            enabled=enabled,
            format_=format_,
            triggered_configuration=triggered_configuration,
            sms_tags=sms_tags,
            device_scope=device_scope,
            last_triggered_time=last_triggered_time,
            event_trigger_group_id=event_trigger_group_id,
            stateful_trigger_configuration=stateful_trigger_configuration,
            is_stateful=is_stateful,
            forwarding_configuration=forwarding_configuration,
            sets_device_color=sets_device_color,
            id=id,
            created_at=created_at,
            updated_at=updated_at,
            tags=tags,
        )

        event_trigger.additional_properties = d
        return event_trigger

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class PartialEmailConfigurationEmailType(str, Enum):
    SIGN_UP = "sign-up"
    PASSWORD_RESET = "password-reset"

    def __str__(self) -> str:
        return str(self.value)




from formant.sdk.cloud.v2.formant_admin_api_client.models import Camera, NumericViewConfiguration, Location, BitsetViewConfiguration, PartialViewTags, LocalizationViewConfiguration, LocationViewport, ImageViewConfigurationMode, LocationViewConfigurationBasemap, ViewConfigurationType, ViewConfiguration, PartialViewLayoutType, Filter, View, PointCloudViewConfiguration, ViewLayoutType, LocationViewConfiguration, ImageViewConfiguration, ViewListResponse, TransformTreeViewConfiguration, ViewTags, Vector3, PartialView
from formant.sdk.cloud.v2.formant_admin_api_client.api.view import view_controller_get_one, view_controller_get_all, view_controller_patch
from formant.sdk.cloud.v2.formant_admin_api_client.types import *



from typing import Any, Dict, List, Type, TypeVar

import attr

from ..models.named_json_schema_schema_type import NamedJsonSchemaSchemaType

T = TypeVar("T", bound="NamedJsonSchema")


@attr.s(auto_attribs=True)
class NamedJsonSchema:
    """
    Attributes:
        name (str):
        id (str):
        url (str):
        schema_type (NamedJsonSchemaSchemaType):
    """

    name: str
    id: str
    url: str
    schema_type: NamedJsonSchemaSchemaType
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        id = self.id
        url = self.url
        schema_type = self.schema_type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "id": id,
                "url": url,
                "schemaType": schema_type,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        id = d.pop("id")

        url = d.pop("url")

        schema_type = NamedJsonSchemaSchemaType(d.pop("schemaType"))

        named_json_schema = cls(
            name=name,
            id=id,
            url=url,
            schema_type=schema_type,
        )

        named_json_schema.additional_properties = d
        return named_json_schema

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.create_event_trigger_group_request_tags import CreateEventTriggerGroupRequestTags
    from ..models.event_trigger import EventTrigger


T = TypeVar("T", bound="CreateEventTriggerGroupRequest")


@attr.s(auto_attribs=True)
class CreateEventTriggerGroupRequest:
    """
    Attributes:
        phone_numbers (List[str]):
        device_ids (List[str]):
        event_triggers (EventTrigger):
        tags (Union[Unset, CreateEventTriggerGroupRequestTags]):
        is_already_opted_in (Union[Unset, bool]):
    """

    phone_numbers: List[str]
    device_ids: List[str]
    event_triggers: "EventTrigger"
    tags: Union[Unset, "CreateEventTriggerGroupRequestTags"] = UNSET
    is_already_opted_in: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        phone_numbers = self.phone_numbers

        device_ids = self.device_ids

        event_triggers = self.event_triggers.to_dict()

        tags: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = self.tags.to_dict()

        is_already_opted_in = self.is_already_opted_in

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "phoneNumbers": phone_numbers,
                "deviceIds": device_ids,
                "eventTriggers": event_triggers,
            }
        )
        if tags is not UNSET:
            field_dict["tags"] = tags
        if is_already_opted_in is not UNSET:
            field_dict["isAlreadyOptedIn"] = is_already_opted_in

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.create_event_trigger_group_request_tags import CreateEventTriggerGroupRequestTags
        from ..models.event_trigger import EventTrigger

        d = src_dict.copy()
        phone_numbers = cast(List[str], d.pop("phoneNumbers"))

        device_ids = cast(List[str], d.pop("deviceIds"))

        event_triggers = EventTrigger.from_dict(d.pop("eventTriggers"))

        _tags = d.pop("tags", UNSET)
        tags: Union[Unset, CreateEventTriggerGroupRequestTags]
        if isinstance(_tags, Unset):
            tags = UNSET
        else:
            tags = CreateEventTriggerGroupRequestTags.from_dict(_tags)

        is_already_opted_in = d.pop("isAlreadyOptedIn", UNSET)

        create_event_trigger_group_request = cls(
            phone_numbers=phone_numbers,
            device_ids=device_ids,
            event_triggers=event_triggers,
            tags=tags,
            is_already_opted_in=is_already_opted_in,
        )

        create_event_trigger_group_request.additional_properties = d
        return create_event_trigger_group_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class EventTriggerSeverity(str, Enum):
    INFO = "info"
    WARNING = "warning"
    ERROR = "error"
    CRITICAL = "critical"

    def __str__(self) -> str:
        return str(self.value)



import unittest
from formant.sdk.cloud.v2.src.admin_api import AdminAPI
from formant.sdk.cloud.v2.formant_admin_api_client.models import PartialView
import os

EMAIL = os.getenv("FORMANT_EMAIL")
PASSWORD = os.getenv("FORMANT_PASSWORD")
BASE_URL = "https://api.formant.io/v1"


class TestViews(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super(TestViews, self).__init__(*args, **kwargs)
        self.client = AdminAPI(email=EMAIL, password=PASSWORD, base_url=BASE_URL)

    def test_get(self):
        view_id = "d1201530-a65e-4e6a-948a-6af0a368d4b3"
        result = self.client.views.get(view_id)
        self.assertEqual(result.status_code, 200)

    def test_get_all(self):
        result = self.client.views.get_all()
        self.assertEqual(result.status_code, 200)

    def test_patch(self):
        view_id = "d1201530-a65e-4e6a-948a-6af0a368d4b3"
        partial_view = PartialView(organization_id=self.client.organization_id)
        result = self.client.views.patch(view_id, partial_view)
        self.assertEqual(result.status_code, 200)


unittest.main()



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.device_application_configuration_configuration_map import (
        DeviceApplicationConfigurationConfigurationMap,
    )


T = TypeVar("T", bound="DeviceApplicationConfiguration")


@attr.s(auto_attribs=True)
class DeviceApplicationConfiguration:
    """
    Attributes:
        configuration_map (Union[Unset, DeviceApplicationConfigurationConfigurationMap]):
    """

    configuration_map: Union[Unset, "DeviceApplicationConfigurationConfigurationMap"] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        configuration_map: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.configuration_map, Unset):
            configuration_map = self.configuration_map.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if configuration_map is not UNSET:
            field_dict["configurationMap"] = configuration_map

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.device_application_configuration_configuration_map import (
            DeviceApplicationConfigurationConfigurationMap,
        )

        d = src_dict.copy()
        _configuration_map = d.pop("configurationMap", UNSET)
        configuration_map: Union[Unset, DeviceApplicationConfigurationConfigurationMap]
        if isinstance(_configuration_map, Unset):
            configuration_map = UNSET
        else:
            configuration_map = DeviceApplicationConfigurationConfigurationMap.from_dict(_configuration_map)

        device_application_configuration = cls(
            configuration_map=configuration_map,
        )

        device_application_configuration.additional_properties = d
        return device_application_configuration

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class DatapointEventStreamType(str, Enum):
    BITSET = "bitset"
    LOCALIZATION = "localization"
    POINT_CLOUD = "point cloud"
    LOCATION = "location"
    FILE = "file"
    HEALTH = "health"
    TRANSFORM_TREE = "transform tree"
    BATTERY = "battery"
    VIDEO = "video"
    NUMERIC_SET = "numeric set"
    JSON = "json"
    IMAGE = "image"
    NUMERIC = "numeric"
    TEXT = "text"

    def __str__(self) -> str:
        return str(self.value)



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..models.stream_stream_type import StreamStreamType
from ..types import UNSET, Unset

T = TypeVar("T", bound="Stream")


@attr.s(auto_attribs=True)
class Stream:
    """
    Attributes:
        stream_name (str):
        stream_type (StreamStreamType):
        organization_id (Union[Unset, str]):
        description (Union[Unset, None, str]):
        alias (Union[Unset, None, str]):
        is_telemetry_filter (Union[Unset, bool]):
        is_event_filter (Union[Unset, bool]):
        active (Union[Unset, bool]):
        is_overview_column (Union[Unset, bool]):
        is_overview_row (Union[Unset, bool]):
        enabled (Union[Unset, bool]):
        id (Union[Unset, str]):
        created_at (Union[Unset, datetime.datetime]):
        updated_at (Union[Unset, datetime.datetime]):
    """

    stream_name: str
    stream_type: StreamStreamType
    organization_id: Union[Unset, str] = UNSET
    description: Union[Unset, None, str] = UNSET
    alias: Union[Unset, None, str] = UNSET
    is_telemetry_filter: Union[Unset, bool] = UNSET
    is_event_filter: Union[Unset, bool] = UNSET
    active: Union[Unset, bool] = UNSET
    is_overview_column: Union[Unset, bool] = UNSET
    is_overview_row: Union[Unset, bool] = UNSET
    enabled: Union[Unset, bool] = UNSET
    id: Union[Unset, str] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    updated_at: Union[Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        stream_name = self.stream_name
        stream_type = self.stream_type.value

        organization_id = self.organization_id
        description = self.description
        alias = self.alias
        is_telemetry_filter = self.is_telemetry_filter
        is_event_filter = self.is_event_filter
        active = self.active
        is_overview_column = self.is_overview_column
        is_overview_row = self.is_overview_row
        enabled = self.enabled
        id = self.id
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        updated_at: Union[Unset, str] = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "streamName": stream_name,
                "streamType": stream_type,
            }
        )
        if organization_id is not UNSET:
            field_dict["organizationId"] = organization_id
        if description is not UNSET:
            field_dict["description"] = description
        if alias is not UNSET:
            field_dict["alias"] = alias
        if is_telemetry_filter is not UNSET:
            field_dict["isTelemetryFilter"] = is_telemetry_filter
        if is_event_filter is not UNSET:
            field_dict["isEventFilter"] = is_event_filter
        if active is not UNSET:
            field_dict["active"] = active
        if is_overview_column is not UNSET:
            field_dict["isOverviewColumn"] = is_overview_column
        if is_overview_row is not UNSET:
            field_dict["isOverviewRow"] = is_overview_row
        if enabled is not UNSET:
            field_dict["enabled"] = enabled
        if id is not UNSET:
            field_dict["id"] = id
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if updated_at is not UNSET:
            field_dict["updatedAt"] = updated_at

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        stream_name = d.pop("streamName")

        stream_type = StreamStreamType(d.pop("streamType"))

        organization_id = d.pop("organizationId", UNSET)

        description = d.pop("description", UNSET)

        alias = d.pop("alias", UNSET)

        is_telemetry_filter = d.pop("isTelemetryFilter", UNSET)

        is_event_filter = d.pop("isEventFilter", UNSET)

        active = d.pop("active", UNSET)

        is_overview_column = d.pop("isOverviewColumn", UNSET)

        is_overview_row = d.pop("isOverviewRow", UNSET)

        enabled = d.pop("enabled", UNSET)

        id = d.pop("id", UNSET)

        _created_at = d.pop("createdAt", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        _updated_at = d.pop("updatedAt", UNSET)
        updated_at: Union[Unset, datetime.datetime]
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        stream = cls(
            stream_name=stream_name,
            stream_type=stream_type,
            organization_id=organization_id,
            description=description,
            alias=alias,
            is_telemetry_filter=is_telemetry_filter,
            is_event_filter=is_event_filter,
            active=active,
            is_overview_column=is_overview_column,
            is_overview_row=is_overview_row,
            enabled=enabled,
            id=id,
            created_at=created_at,
            updated_at=updated_at,
        )

        stream.additional_properties = d
        return stream

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar

import attr

from ..models.json_event_trigger_condition_type import JsonEventTriggerConditionType

T = TypeVar("T", bound="JsonEventTriggerCondition")


@attr.s(auto_attribs=True)
class JsonEventTriggerCondition:
    """
    Attributes:
        type (JsonEventTriggerConditionType):
        jq_query (str):
        stream (str):
    """

    type: JsonEventTriggerConditionType
    jq_query: str
    stream: str
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type = self.type.value

        jq_query = self.jq_query
        stream = self.stream

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type,
                "jqQuery": jq_query,
                "stream": stream,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        type = JsonEventTriggerConditionType(d.pop("type"))

        jq_query = d.pop("jqQuery")

        stream = d.pop("stream")

        json_event_trigger_condition = cls(
            type=type,
            jq_query=jq_query,
            stream=stream,
        )

        json_event_trigger_condition.additional_properties = d
        return json_event_trigger_condition

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from formant.protos.model.v1 import math_pb2
from formant.sdk.agent.v1.localization.utils import validate_type, get_ros_module
from .quaternion import Quaternion
from .vector3 import Vector3


class Transform:
    @classmethod
    def from_ros_transform(cls, transform):
        msgs = get_ros_module("geometry_msgs.msg")
        validate_type(transform, msgs.Transform)
        return cls(
            translation=Vector3.from_ros(transform.translation),
            rotation=Quaternion.from_ros(transform.rotation),
        )

    @classmethod
    def from_ros_transform_stamped(cls, transform_stamped):
        msgs = get_ros_module("geometry_msgs.msg")
        validate_type(transform_stamped, msgs.TransformStamped)
        return cls.from_ros_transform(transform_stamped.transform)

    @classmethod
    def from_ros_pose_stamped(cls, pose_stamped):
        msgs = get_ros_module("geometry_msgs.msg")
        validate_type(pose_stamped, msgs.PoseStamped)
        return cls.from_ros_pose(pose_stamped.pose)

    @classmethod
    def from_ros_pose_with_covariance(cls, pose_with_covariance):
        msgs = get_ros_module("geometry_msgs.msg")
        validate_type(pose_with_covariance, msgs.PoseWithCovariance)
        return cls.from_ros_pose(pose_with_covariance.pose)

    @classmethod
    def from_ros_pose(cls, pose):
        msgs = get_ros_module("geometry_msgs.msg")
        validate_type(pose, msgs.Pose)
        return cls(
            translation=Vector3.from_ros_point(pose.position),
            rotation=Quaternion.from_ros(pose.orientation),
        )

    @classmethod
    def from_xy_direction(
        cls,
        x,  # type: float
        y,  # type: float
        direction,  # type: float
    ):
        return cls(
            translation=Vector3.from_xy(x, y),
            rotation=Quaternion.from_direction(direction),
        )

    def __init__(
        self,
        translation=Vector3.identity(),  # type: Vector3
        rotation=Quaternion.identity(),  # type: Quaternion
    ):
        self.translation = translation
        self.rotation = rotation

    def to_proto(self):
        return math_pb2.Transform(
            translation=self.translation.to_proto(), rotation=self.rotation.to_proto()
        )



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.device_ros_configuration import DeviceRosConfiguration
    from ..models.device_stream_configuration import DeviceStreamConfiguration


T = TypeVar("T", bound="DeviceTelemetryConfiguration")


@attr.s(auto_attribs=True)
class DeviceTelemetryConfiguration:
    """
    Attributes:
        streams (Union[Unset, List['DeviceStreamConfiguration']]):
        ros (Union[Unset, DeviceRosConfiguration]):
    """

    streams: Union[Unset, List["DeviceStreamConfiguration"]] = UNSET
    ros: Union[Unset, "DeviceRosConfiguration"] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        streams: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.streams, Unset):
            streams = []
            for streams_item_data in self.streams:
                streams_item = streams_item_data.to_dict()

                streams.append(streams_item)

        ros: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.ros, Unset):
            ros = self.ros.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if streams is not UNSET:
            field_dict["streams"] = streams
        if ros is not UNSET:
            field_dict["ros"] = ros

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.device_ros_configuration import DeviceRosConfiguration
        from ..models.device_stream_configuration import DeviceStreamConfiguration

        d = src_dict.copy()
        streams = []
        _streams = d.pop("streams", UNSET)
        for streams_item_data in _streams or []:
            streams_item = DeviceStreamConfiguration.from_dict(streams_item_data)

            streams.append(streams_item)

        _ros = d.pop("ros", UNSET)
        ros: Union[Unset, DeviceRosConfiguration]
        if isinstance(_ros, Unset):
            ros = UNSET
        else:
            ros = DeviceRosConfiguration.from_dict(_ros)

        device_telemetry_configuration = cls(
            streams=streams,
            ros=ros,
        )

        device_telemetry_configuration.additional_properties = d
        return device_telemetry_configuration

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



""" Contains all the data models used in inputs/outputs """

from .active_devices_query import ActiveDevicesQuery
from .active_devices_query_type import ActiveDevicesQueryType
from .battery import Battery
from .bitset import Bitset
from .bounding_box import BoundingBox
from .file import File
from .goal import Goal
from .health import Health
from .health_status import HealthStatus
from .image import Image
from .image_annotation import ImageAnnotation
from .interval_query import IntervalQuery
from .interval_query_aggregate import IntervalQueryAggregate
from .interval_query_interval import IntervalQueryInterval
from .interval_query_types_item import IntervalQueryTypesItem
from .iso_date_list_response import IsoDateListResponse
from .last_seen_response import LastSeenResponse
from .localization import Localization
from .location import Location
from .map_ import Map
from .metadata import Metadata
from .metadata_list_response import MetadataListResponse
from .metadata_tags import MetadataTags
from .metadata_type import MetadataType
from .numeric_set_entry import NumericSetEntry
from .odometry import Odometry
from .path import Path
from .point_cloud import PointCloud
from .quaternion import Quaternion
from .query import Query
from .query_aggregate import QueryAggregate
from .query_by_device_request import QueryByDeviceRequest
from .query_types_item import QueryTypesItem
from .scope_filter import ScopeFilter
from .scope_filter_types_item import ScopeFilterTypesItem
from .seek_query import SeekQuery
from .seek_query_direction import SeekQueryDirection
from .seek_query_types_item import SeekQueryTypesItem
from .seek_result import SeekResult
from .sql_column import SqlColumn
from .sql_query import SqlQuery
from .sql_query_aggregate_level import SqlQueryAggregateLevel
from .sql_query_aggregate_type import SqlQueryAggregateType
from .sql_query_types_item import SqlQueryTypesItem
from .sql_result import SqlResult
from .sql_table import SqlTable
from .sql_tables_list_reponse import SqlTablesListReponse
from .stream_aggregate_data import StreamAggregateData
from .stream_aggregate_data_tags import StreamAggregateDataTags
from .stream_aggregate_data_type import StreamAggregateDataType
from .stream_column_list_response import StreamColumnListResponse
from .stream_current_value_list_response import StreamCurrentValueListResponse
from .stream_data import StreamData
from .stream_data_list_response import StreamDataListResponse
from .stream_data_points_item import StreamDataPointsItem
from .stream_data_tags import StreamDataTags
from .stream_data_type import StreamDataType
from .string_list_response import StringListResponse
from .task_report_column_list_response import TaskReportColumnListResponse
from .telemetry_export_sheet_request import TelemetryExportSheetRequest
from .telemetry_export_sheet_result import TelemetryExportSheetResult
from .transform import Transform
from .transform_node import TransformNode
from .twist import Twist
from .usage_metric import UsageMetric
from .usage_metric_metric_type import UsageMetricMetricType
from .usage_metrics_query import UsageMetricsQuery
from .usage_metrics_query_metric_type import UsageMetricsQueryMetricType
from .usage_metrics_query_response import UsageMetricsQueryResponse
from .uuid_list_response import UuidListResponse
from .vector_3 import Vector3
from .video import Video
from .video_mime_type import VideoMimeType

__all__ = (
    "ActiveDevicesQuery",
    "ActiveDevicesQueryType",
    "Battery",
    "Bitset",
    "BoundingBox",
    "File",
    "Goal",
    "Health",
    "HealthStatus",
    "Image",
    "ImageAnnotation",
    "IntervalQuery",
    "IntervalQueryAggregate",
    "IntervalQueryInterval",
    "IntervalQueryTypesItem",
    "IsoDateListResponse",
    "LastSeenResponse",
    "Localization",
    "Location",
    "Map",
    "Metadata",
    "MetadataListResponse",
    "MetadataTags",
    "MetadataType",
    "NumericSetEntry",
    "Odometry",
    "Path",
    "PointCloud",
    "Quaternion",
    "Query",
    "QueryAggregate",
    "QueryByDeviceRequest",
    "QueryTypesItem",
    "ScopeFilter",
    "ScopeFilterTypesItem",
    "SeekQuery",
    "SeekQueryDirection",
    "SeekQueryTypesItem",
    "SeekResult",
    "SqlColumn",
    "SqlQuery",
    "SqlQueryAggregateLevel",
    "SqlQueryAggregateType",
    "SqlQueryTypesItem",
    "SqlResult",
    "SqlTable",
    "SqlTablesListReponse",
    "StreamAggregateData",
    "StreamAggregateDataTags",
    "StreamAggregateDataType",
    "StreamColumnListResponse",
    "StreamCurrentValueListResponse",
    "StreamData",
    "StreamDataListResponse",
    "StreamDataPointsItem",
    "StreamDataTags",
    "StreamDataType",
    "StringListResponse",
    "TaskReportColumnListResponse",
    "TelemetryExportSheetRequest",
    "TelemetryExportSheetResult",
    "Transform",
    "TransformNode",
    "Twist",
    "UsageMetric",
    "UsageMetricMetricType",
    "UsageMetricsQuery",
    "UsageMetricsQueryMetricType",
    "UsageMetricsQueryResponse",
    "UuidListResponse",
    "Vector3",
    "Video",
    "VideoMimeType",
)




from . import Response, View, ViewListResponse, PartialView, view_controller_get_one, view_controller_get_all, view_controller_patch
from formant.sdk.cloud.v2.src.resources.resources import Resources

class Views(Resources):

    def get(self, device_id: str):
        'Get a device layout'
        client = self._get_client()
        response: Response[View] = view_controller_get_one.sync_detailed(client=client, id=device_id)
        return response

    async def get_async(self, device_id: str):
        'Get a device layout'
        client = self._get_client()
        response: Response[View] = (await view_controller_get_one.asyncio_detailed(client=client, id=device_id))
        return response

    def get_all(self):
        'List all device layouts'
        client = self._get_client()
        response: Response[ViewListResponse] = view_controller_get_all.sync_detailed(client=client)
        return response

    async def get_all_async(self):
        'List all device layouts'
        client = self._get_client()
        response: Response[ViewListResponse] = (await view_controller_get_all.asyncio_detailed(client=client))
        return response

    def patch(self, id: str, partial_view: PartialView):
        'Update a device layout'
        client = self._get_client()
        response: Response[View] = view_controller_patch.sync_detailed(client=client, id=id, json_body=partial_view)
        return response

    async def patch_async(self, id: str, partial_view: PartialView):
        'Update a device layout'
        client = self._get_client()
        response: Response[View] = (await view_controller_patch.asyncio_detailed(client=client, id=id, json_body=partial_view))
        return response



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.create_intervention_response_intervention_type import CreateInterventionResponseInterventionType
from ..models.create_intervention_response_type import CreateInterventionResponseType
from ..types import UNSET, Unset

T = TypeVar("T", bound="CreateInterventionResponse")


@attr.s(auto_attribs=True)
class CreateInterventionResponse:
    """
    Attributes:
        intervention_type (CreateInterventionResponseInterventionType):
        data (Any):
        intervention_id (str):
        type (Union[Unset, CreateInterventionResponseType]):
        tags (Union[Unset, Any]):
        time (Union[Unset, Any]):
        message (Union[Unset, Any]):
        device_id (Union[Unset, Any]):
        user_id (Union[Unset, str]):
    """

    intervention_type: CreateInterventionResponseInterventionType
    data: Any
    intervention_id: str
    type: Union[Unset, CreateInterventionResponseType] = UNSET
    tags: Union[Unset, Any] = UNSET
    time: Union[Unset, Any] = UNSET
    message: Union[Unset, Any] = UNSET
    device_id: Union[Unset, Any] = UNSET
    user_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        intervention_type = self.intervention_type.value

        data = self.data
        intervention_id = self.intervention_id
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        tags = self.tags
        time = self.time
        message = self.message
        device_id = self.device_id
        user_id = self.user_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "interventionType": intervention_type,
                "data": data,
                "interventionId": intervention_id,
            }
        )
        if type is not UNSET:
            field_dict["type"] = type
        if tags is not UNSET:
            field_dict["tags"] = tags
        if time is not UNSET:
            field_dict["time"] = time
        if message is not UNSET:
            field_dict["message"] = message
        if device_id is not UNSET:
            field_dict["deviceId"] = device_id
        if user_id is not UNSET:
            field_dict["userId"] = user_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        intervention_type = CreateInterventionResponseInterventionType(d.pop("interventionType"))

        data = d.pop("data")

        intervention_id = d.pop("interventionId")

        _type = d.pop("type", UNSET)
        type: Union[Unset, CreateInterventionResponseType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = CreateInterventionResponseType(_type)

        tags = d.pop("tags", UNSET)

        time = d.pop("time", UNSET)

        message = d.pop("message", UNSET)

        device_id = d.pop("deviceId", UNSET)

        user_id = d.pop("userId", UNSET)

        create_intervention_response = cls(
            intervention_type=intervention_type,
            data=data,
            intervention_id=intervention_id,
            type=type,
            tags=tags,
            time=time,
            message=message,
            device_id=device_id,
            user_id=user_id,
        )

        create_intervention_response.additional_properties = d
        return create_intervention_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties




from formant.sdk.cloud.v2.formant_admin_api_client.models import DeviceOnlineEventSeverity, VideoDevice, DeviceProvisioning, AudioDevice, DeviceStreamRosTransformTreeConfiguration, NodeInfo, DeviceStreamConfiguration, TeleopJoystickConfigurationPosition, RtcInfoRtcIceTransportPoliciesItem, DeviceOnlineEventMetadata, DeviceStreamConfigurationTags, DeviceStreamRosTopicConfigurationType, DeviceDetails, OnDemandPresenceStreamItemGroup, DeviceTeleopCustomStreamConfigurationQuality, DeviceTeleopRosStreamConfigurationMode, ValidationConfiguration, TeleopJoystickAxisConfigurationDimension, OnvifDevice, DeviceStreamRosTopicConfiguration, DeviceStreamFileTailConfigurationFileFormat, Device, NetworkInfo, DeviceStreamHardwareConfigurationHardwareType, DeviceTeleopCustomStreamConfiguration, ScopeFilter, DeviceTeleopRosStreamConfigurationTopicType, DeviceRosConfiguration, DeviceOfflineEventType, RtcInfoRtcIceServerProtocol, DeviceOnlineEventStreamType, DeviceStreamFileTailConfigurationType, DeviceDetailsType, OnDemandBufferBufferType, TeleopViewConfiguration, DeviceTeleopHardwareStreamConfigurationQuality, DeviceTeleopRosStreamConfigurationNumericControlVisualization, DeviceQueryType, DeviceListResponse, DeviceTeleopHardwareStreamConfigurationMode, DeviceStreamRosLocalizationConfigurationType, DeviceDiskConfiguration, DeviceStreamRosTransformTreeConfigurationType, DeviceTags, DeviceDetailsListResponse, OnDemandStreamPresence, DeviceStreamHardwareConfiguration, DeviceOfflineEventStreamType, DeviceOnlineEventTags, DeviceTeleopRosStreamConfigurationAudioCodec, ScopeFilterTypesItem, DeviceTeleopHardwareStreamConfigurationRtcStreamType, DeviceProvisioningRequest, OnDemandBuffer, DeviceTeleopHardwareStreamConfigurationHardwareType, DeviceStreamCustomConfiguration, DeviceTeleopCustomStreamConfigurationRtcStreamType, PartialDevice, DeviceTeleopCustomStreamConfigurationNumericControlVisualization, PartialDeviceTags, DeviceTeleopCustomStreamConfigurationMode, TeleopJoystickConfiguration, OnDemandPresenceStreamItemGroupDatapointType, DeviceDetailsTags, DeviceTeleopHardwareStreamConfiguration, DeviceBlobData, CommandProgress, DeviceStateEnv, PartialDeviceType, DeviceReportedConfigurationState, DeviceStreamFileTailConfiguration, DeviceOfflineEventSeverity, DeviceStreamTransformConfiguration, DeviceStreamDirectoryWatchConfiguration, DeviceStreamConfigurationQuality, OsInfo, DevicePortForwardingConfiguration, DeviceRosState, DeviceConfigurationTemplateTags, KernelInfo, DeviceStreamDirectoryWatchConfigurationFileType, DeviceConfigurationTemplate, DeviceConfiguration, DeviceOnlineEventType, RosTopic, TeleopJoystickAxisConfiguration, DeviceTeleopRosStreamConfigurationQuality, DeviceOnlineEvent, DeviceFollower, AdapterConfiguration, DeviceTelemetryConfiguration, DeviceStreamHardwareConfigurationType, DeviceOfflineEvent, OnDemandPresenceTimeRange, DeviceOfflineEventMetadata, DeviceState, DeviceStreamRosLocalizationConfiguration, DeviceConfigurationDocument, DeviceOfflineEventTags, DeviceScope, HwInfo, DeviceApplicationConfiguration, DeviceTeleopRosStreamConfiguration, DeviceStreamDirectoryWatchConfigurationType, OnDemandState, DeviceResourcesConfiguration, DeviceApplicationConfigurationConfigurationMap, DeviceCredentials, DeviceType, DeviceConfigurationDocumentTags, DeviceTeleopConfiguration, DeviceQuery, DeviceConfigurationTemplateListResponse, DeviceStreamCustomConfigurationType, RtcInfo, DeviceStreamHardwareConfigurationQuality, DeviceScopeTypesItem, Network, DeviceDiagnosticsConfiguration
from formant.sdk.cloud.v2.formant_admin_api_client.api.device import device_controller_query, device_controller_patch, device_controller_get_one, device_controller_get_configuration, device_controller_post_configuration
from formant.sdk.cloud.v2.formant_admin_api_client.types import *
from formant.sdk.cloud.v2.formant_admin_api_client.types import Optional



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.device_configuration import DeviceConfiguration


T = TypeVar("T", bound="UpdatedConfigurationResponse")


@attr.s(auto_attribs=True)
class UpdatedConfigurationResponse:
    """
    Attributes:
        desired_configuration (Union[Unset, None, DeviceConfiguration]):
    """

    desired_configuration: Union[Unset, None, "DeviceConfiguration"] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        desired_configuration: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self.desired_configuration, Unset):
            desired_configuration = self.desired_configuration.to_dict() if self.desired_configuration else None

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if desired_configuration is not UNSET:
            field_dict["desiredConfiguration"] = desired_configuration

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.device_configuration import DeviceConfiguration

        d = src_dict.copy()
        _desired_configuration = d.pop("desiredConfiguration", UNSET)
        desired_configuration: Union[Unset, None, DeviceConfiguration]
        if _desired_configuration is None:
            desired_configuration = None
        elif isinstance(_desired_configuration, Unset):
            desired_configuration = UNSET
        else:
            desired_configuration = DeviceConfiguration.from_dict(_desired_configuration)

        updated_configuration_response = cls(
            desired_configuration=desired_configuration,
        )

        updated_configuration_response.additional_properties = d
        return updated_configuration_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties




from formant.sdk.cloud.v2.formant_admin_api_client.models import CommandTemplate, ScopeFilter, CommandParameter, CommandTemplateParameterMeta, ScopeFilterTypesItem, CommandResponse, CommandTags, CommandProgress, CommandListResponse, CommandDeliveryEventType, CommandTemplateTags, CommandDeliveryEventMetadata, CommandResponseTags, CommandParameterMeta, CommandDeliveryEventStreamType, CommandResponseStreamType, FileInfo, CommandRequest, Command, CommandDeliveryEvent, CommandTemplateListResponse, CommandStreamType, CommandRequestListResponse, CommandQuery, CommandDeliveryEventTags
from formant.sdk.cloud.v2.formant_admin_api_client.api.command import command_controller_post, command_controller_query
from formant.sdk.cloud.v2.formant_admin_api_client.types import *



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="File")


@attr.s(auto_attribs=True)
class File:
    """
    Attributes:
        url (str):
        filename (str):
        size (int):
        preview_url (Union[Unset, str]):
    """

    url: str
    filename: str
    size: int
    preview_url: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        url = self.url
        filename = self.filename
        size = self.size
        preview_url = self.preview_url

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "url": url,
                "filename": filename,
                "size": size,
            }
        )
        if preview_url is not UNSET:
            field_dict["previewUrl"] = preview_url

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        url = d.pop("url")

        filename = d.pop("filename")

        size = d.pop("size")

        preview_url = d.pop("previewUrl", UNSET)

        file = cls(
            url=url,
            filename=filename,
            size=size,
            preview_url=preview_url,
        )

        file.additional_properties = d
        return file

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class HealthStatus(str, Enum):
    UNKNOWN = "unknown"
    OPERATIONAL = "operational"
    OFFLINE = "offline"
    ERROR = "error"

    def __str__(self) -> str:
        return str(self.value)



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

import attr
from dateutil.parser import isoparse

from ..models.device_type import DeviceType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.device_follower import DeviceFollower
    from ..models.device_scope import DeviceScope
    from ..models.device_state import DeviceState
    from ..models.device_tags import DeviceTags
    from ..models.scope_filter import ScopeFilter


T = TypeVar("T", bound="Device")


@attr.s(auto_attribs=True)
class Device:
    """
    Attributes:
        name (str): Name of this device.
        public_key (str):
        organization_id (Union[Unset, str]): ID of the organization which contains this device.
        description (Union[Unset, None, str]):
        type (Union[Unset, DeviceType]):
        user_id (Union[Unset, None, str]):
        fleet_id (Union[Unset, None, str]): ID of the fleet with which this device is associated.
        scope (Union[Unset, None, ScopeFilter]):
        device_scope (Union[Unset, DeviceScope]):
        desired_agent_version (Union[Unset, None, str]):
        desired_configuration_version (Union[Unset, None, int]):
        temporary_configuration_version (Union[Unset, None, int]):
        temporary_configuration_expiration (Union[Unset, None, datetime.datetime]):
        temporary_configuration_template_id (Union[Unset, None, str]):
        followers (Union[Unset, List['DeviceFollower']]): Users who receive SMS updates from this device.
        slack_channels (Union[Unset, List[str]]): Slack channels which receive updates from this device.
        state (Union[Unset, DeviceState]):
        enabled (Union[Unset, bool]):
        fully_configured (Union[Unset, bool]):
        disabled_at (Union[Unset, None, datetime.datetime]):
        id (Union[Unset, str]):
        created_at (Union[Unset, datetime.datetime]):
        updated_at (Union[Unset, datetime.datetime]):
        tags (Union[Unset, DeviceTags]):
    """

    name: str
    public_key: str
    organization_id: Union[Unset, str] = UNSET
    description: Union[Unset, None, str] = UNSET
    type: Union[Unset, DeviceType] = UNSET
    user_id: Union[Unset, None, str] = UNSET
    fleet_id: Union[Unset, None, str] = UNSET
    scope: Union[Unset, None, "ScopeFilter"] = UNSET
    device_scope: Union[Unset, "DeviceScope"] = UNSET
    desired_agent_version: Union[Unset, None, str] = UNSET
    desired_configuration_version: Union[Unset, None, int] = UNSET
    temporary_configuration_version: Union[Unset, None, int] = UNSET
    temporary_configuration_expiration: Union[Unset, None, datetime.datetime] = UNSET
    temporary_configuration_template_id: Union[Unset, None, str] = UNSET
    followers: Union[Unset, List["DeviceFollower"]] = UNSET
    slack_channels: Union[Unset, List[str]] = UNSET
    state: Union[Unset, "DeviceState"] = UNSET
    enabled: Union[Unset, bool] = UNSET
    fully_configured: Union[Unset, bool] = UNSET
    disabled_at: Union[Unset, None, datetime.datetime] = UNSET
    id: Union[Unset, str] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    updated_at: Union[Unset, datetime.datetime] = UNSET
    tags: Union[Unset, "DeviceTags"] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        public_key = self.public_key
        organization_id = self.organization_id
        description = self.description
        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        user_id = self.user_id
        fleet_id = self.fleet_id
        scope: Union[Unset, None, Dict[str, Any]] = UNSET
        if not isinstance(self.scope, Unset):
            scope = self.scope.to_dict() if self.scope else None

        device_scope: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.device_scope, Unset):
            device_scope = self.device_scope.to_dict()

        desired_agent_version = self.desired_agent_version
        desired_configuration_version = self.desired_configuration_version
        temporary_configuration_version = self.temporary_configuration_version
        temporary_configuration_expiration: Union[Unset, None, str] = UNSET
        if not isinstance(self.temporary_configuration_expiration, Unset):
            temporary_configuration_expiration = (
                self.temporary_configuration_expiration.isoformat() if self.temporary_configuration_expiration else None
            )

        temporary_configuration_template_id = self.temporary_configuration_template_id
        followers: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.followers, Unset):
            followers = []
            for followers_item_data in self.followers:
                followers_item = followers_item_data.to_dict()

                followers.append(followers_item)

        slack_channels: Union[Unset, List[str]] = UNSET
        if not isinstance(self.slack_channels, Unset):
            slack_channels = self.slack_channels

        state: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.state, Unset):
            state = self.state.to_dict()

        enabled = self.enabled
        fully_configured = self.fully_configured
        disabled_at: Union[Unset, None, str] = UNSET
        if not isinstance(self.disabled_at, Unset):
            disabled_at = self.disabled_at.isoformat() if self.disabled_at else None

        id = self.id
        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        updated_at: Union[Unset, str] = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()

        tags: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = self.tags.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "publicKey": public_key,
            }
        )
        if organization_id is not UNSET:
            field_dict["organizationId"] = organization_id
        if description is not UNSET:
            field_dict["description"] = description
        if type is not UNSET:
            field_dict["type"] = type
        if user_id is not UNSET:
            field_dict["userId"] = user_id
        if fleet_id is not UNSET:
            field_dict["fleetId"] = fleet_id
        if scope is not UNSET:
            field_dict["scope"] = scope
        if device_scope is not UNSET:
            field_dict["deviceScope"] = device_scope
        if desired_agent_version is not UNSET:
            field_dict["desiredAgentVersion"] = desired_agent_version
        if desired_configuration_version is not UNSET:
            field_dict["desiredConfigurationVersion"] = desired_configuration_version
        if temporary_configuration_version is not UNSET:
            field_dict["temporaryConfigurationVersion"] = temporary_configuration_version
        if temporary_configuration_expiration is not UNSET:
            field_dict["temporaryConfigurationExpiration"] = temporary_configuration_expiration
        if temporary_configuration_template_id is not UNSET:
            field_dict["temporaryConfigurationTemplateId"] = temporary_configuration_template_id
        if followers is not UNSET:
            field_dict["followers"] = followers
        if slack_channels is not UNSET:
            field_dict["slackChannels"] = slack_channels
        if state is not UNSET:
            field_dict["state"] = state
        if enabled is not UNSET:
            field_dict["enabled"] = enabled
        if fully_configured is not UNSET:
            field_dict["fullyConfigured"] = fully_configured
        if disabled_at is not UNSET:
            field_dict["disabledAt"] = disabled_at
        if id is not UNSET:
            field_dict["id"] = id
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if updated_at is not UNSET:
            field_dict["updatedAt"] = updated_at
        if tags is not UNSET:
            field_dict["tags"] = tags

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.device_follower import DeviceFollower
        from ..models.device_scope import DeviceScope
        from ..models.device_state import DeviceState
        from ..models.device_tags import DeviceTags
        from ..models.scope_filter import ScopeFilter

        d = src_dict.copy()
        name = d.pop("name")

        public_key = d.pop("publicKey")

        organization_id = d.pop("organizationId", UNSET)

        description = d.pop("description", UNSET)

        _type = d.pop("type", UNSET)
        type: Union[Unset, DeviceType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = DeviceType(_type)

        user_id = d.pop("userId", UNSET)

        fleet_id = d.pop("fleetId", UNSET)

        _scope = d.pop("scope", UNSET)
        scope: Union[Unset, None, ScopeFilter]
        if _scope is None:
            scope = None
        elif isinstance(_scope, Unset):
            scope = UNSET
        else:
            scope = ScopeFilter.from_dict(_scope)

        _device_scope = d.pop("deviceScope", UNSET)
        device_scope: Union[Unset, DeviceScope]
        if isinstance(_device_scope, Unset):
            device_scope = UNSET
        else:
            device_scope = DeviceScope.from_dict(_device_scope)

        desired_agent_version = d.pop("desiredAgentVersion", UNSET)

        desired_configuration_version = d.pop("desiredConfigurationVersion", UNSET)

        temporary_configuration_version = d.pop("temporaryConfigurationVersion", UNSET)

        _temporary_configuration_expiration = d.pop("temporaryConfigurationExpiration", UNSET)
        temporary_configuration_expiration: Union[Unset, None, datetime.datetime]
        if _temporary_configuration_expiration is None:
            temporary_configuration_expiration = None
        elif isinstance(_temporary_configuration_expiration, Unset):
            temporary_configuration_expiration = UNSET
        else:
            temporary_configuration_expiration = isoparse(_temporary_configuration_expiration)

        temporary_configuration_template_id = d.pop("temporaryConfigurationTemplateId", UNSET)

        followers = []
        _followers = d.pop("followers", UNSET)
        for followers_item_data in _followers or []:
            followers_item = DeviceFollower.from_dict(followers_item_data)

            followers.append(followers_item)

        slack_channels = cast(List[str], d.pop("slackChannels", UNSET))

        _state = d.pop("state", UNSET)
        state: Union[Unset, DeviceState]
        if isinstance(_state, Unset):
            state = UNSET
        else:
            state = DeviceState.from_dict(_state)

        enabled = d.pop("enabled", UNSET)

        fully_configured = d.pop("fullyConfigured", UNSET)

        _disabled_at = d.pop("disabledAt", UNSET)
        disabled_at: Union[Unset, None, datetime.datetime]
        if _disabled_at is None:
            disabled_at = None
        elif isinstance(_disabled_at, Unset):
            disabled_at = UNSET
        else:
            disabled_at = isoparse(_disabled_at)

        id = d.pop("id", UNSET)

        _created_at = d.pop("createdAt", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        _updated_at = d.pop("updatedAt", UNSET)
        updated_at: Union[Unset, datetime.datetime]
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        _tags = d.pop("tags", UNSET)
        tags: Union[Unset, DeviceTags]
        if isinstance(_tags, Unset):
            tags = UNSET
        else:
            tags = DeviceTags.from_dict(_tags)

        device = cls(
            name=name,
            public_key=public_key,
            organization_id=organization_id,
            description=description,
            type=type,
            user_id=user_id,
            fleet_id=fleet_id,
            scope=scope,
            device_scope=device_scope,
            desired_agent_version=desired_agent_version,
            desired_configuration_version=desired_configuration_version,
            temporary_configuration_version=temporary_configuration_version,
            temporary_configuration_expiration=temporary_configuration_expiration,
            temporary_configuration_template_id=temporary_configuration_template_id,
            followers=followers,
            slack_channels=slack_channels,
            state=state,
            enabled=enabled,
            fully_configured=fully_configured,
            disabled_at=disabled_at,
            id=id,
            created_at=created_at,
            updated_at=updated_at,
            tags=tags,
        )

        device.additional_properties = d
        return device

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class DeviceTeleopRosStreamConfigurationNumericControlVisualization(str, Enum):
    SLIDER = "slider"
    DIAL = "dial"

    def __str__(self) -> str:
        return str(self.value)
